<?php

use function apexl\io\env;

return [
    "s3" => [
        "version" => env('MIRROR_S3_VERSION', 'latest'),
        "region" => env('MIRROR_S3_REGION', 'ams3'),
        "endpoint" => env('MIRROR_S3_ENDPOINT', 'https://ams3.digitaloceanspaces.com'),
        "use_path_style_endpoint" => env('MIRROR_S3_USE_PATH_STYLE_ENDPOINT', false),
        "credentials" => [
            "key" => env('MIRROR_S3_KEY'),
            "secret" => env('MIRROR_S3_SECRET'),
        ],
        "bucket" => env('MIRROR_S3_BUCKET'),
    ]
];
