<?php

declare(strict_types=1);

namespace apexl\Io\modules\mirrorS3\Dto;

class S3ConfigDto
{
    public function __construct(
        public string $bucket,
        public string $version,
        public string $endpoint,
        public bool $use_path_style_endpoint,
        public string $key,
        public string $secret,
    ) {}
}
